function [t,y] = odeRK4(odefun,tspan,y0,h)
%odeEuler Solve ordinary differential equations using the Runge-Kutta method.
%
% SYNOPSIS
%    [t,y] = odeRK4(odefun,tspan,y0,h)
%
% INPUT
%   odefun : The f(t,y) function of the y'= f(t,y) differential equations
%   tspan  : Initial (tspan(1)) and stopping value (tspan(2)) of the integration procedure
%   y0     : Initial condition for the dependent variable
%   h      : stepsize
%
% OUTPUT
%   t : vector of independent variable value
%   y : vector of dependent variable value


t = (tspan(1):h:tspan(2))';   % preallocate independent elements
n = length(t);                % total number of elements
y = zeros(n,1);               % preallocate dependent elements
y(1) = y0;                    % set initial value
h2 = h/2; h3 = h/3; h6 = h/6; % avoid repeated evaluation of constants

% Main loop
for j=1:n-1
   k1 = feval(odefun,t(j),y(j));
   k2 = feval(odefun,t(j)+h2,y(j)+h2*k1);
   k3 = feval(odefun,t(j)+h2,y(j)+h2*k2);
   k4 = feval(odefun,t(j)+h,y(j)+h*k3);
   y(j+1) = y(j)+h6*(k1+k4)+h3*(k2+k3);
end
