function [t,y] = odeRK4sys(odefun,tspan,y0,h)
%odeEuler Solve systems of ordinary differential equations using the Runge-Kutta method.
%
% SYNOPSIS
%    [t,y] = odeRK4sys(odefun,tspan,y0,h)
%
% INPUT
%   odefun : The f(t,y) function of the y'= f(t,y) differential equations
%   tspan  : Initial (tspan(1)) and stopping value (tspan(2)) of the integration procedure
%   y0     : Vector of initial condition for the dependent variables
%   h      : stepsize
%
% OUTPUT
%   t : vector of independent variable value
%   y : vector of dependent variable value


t = (tspan(1):h:tspan(2))';   % preallocate independent elements
n = length(t);                % total number of steps
neq = length(y0);             % number of equations that are simultaneously advanced
y = zeros(n,neq);             % preallocate dependent elements
y(1,:) = y0(:)';              % set initial value

h2 = h/2; h3 = h/3; h6 = h/6; % avoid repeated evaluation of constants
k1 = zeros(neq,1);            % preallocate memory for the Runge-Kutta method
k2 = k1;
k3 = k1;
k4 = k1;

% Main loop over time-step
for j=1:n-1
   if mod(j,100)==1
      disp(['#iter: ',num2str(j),'/',num2str(n-1)]);
   end
   told = t(j);              % old time value
   yold = y(j,:)';           % old y values
   k1 = feval(odefun,told,yold)';
   k2 = feval(odefun,told+h2,yold+h2*k1')';
   k3 = feval(odefun,told+h2,yold+h2*k2')';
   k4 = feval(odefun,told+h,yold+h*k3')';
   y(j+1,:) = y(j,:)+h6*(k1+k4)+h3*(k2+k3);
end
